@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -24,7 +24,10 @@ Monkey Patch and feature map for Python Paramiko
 
 import paramiko
 import re
-from paramiko.config import SSH_PORT
+try:
+    from paramiko.config import SSH_PORT
+except ImportError:
+    SSH_PORT=22
 import platform
 from utils import compare_versions
 
