@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -37,6 +37,8 @@ import re
 import locale
 import threading
 import cStringIO
+import base64
+import uuid
 
 from gevent import socket
 
@@ -271,6 +273,13 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         """
         self.disconnect()
 
+    def test_sftpclient(self):
+        ssh_transport = self.get_transport()
+        try:
+            self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
+        except paramiko.SFTPError:
+            raise x2go_exceptions.X2GoSFTPClientException('failed to initialize SFTP channel')
+
     def _x2go_sftp_put(self, local_path, remote_path):
         """
         Put a local file on the remote server via sFTP.
@@ -289,7 +298,10 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         self._transport_lock.acquire()
         if ssh_transport and ssh_transport.is_authenticated():
             self.logger('sFTP-put: %s -> %s:%s' % (os.path.normpath(local_path), self.remote_peername(), remote_path), loglevel=log.loglevel_DEBUG)
-            self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
+            try:
+                self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
+            except paramiko.SFTPError:
+                raise x2go_exceptions.X2GoSFTPClientException('failed to initialize SFTP channel')
             try:
                 self.sftp_client.put(os.path.normpath(local_path), remote_path)
             except (x2go_exceptions.SSHException, socket.error, IOError):
@@ -318,7 +330,10 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         self._transport_lock.acquire()
         if ssh_transport and ssh_transport.is_authenticated():
             self.logger('sFTP-write: opening remote file %s on host %s for writing' % (remote_path, self.remote_peername()), loglevel=log.loglevel_DEBUG)
-            self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
+            try:
+                self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
+            except paramiko.SFTPError:
+                raise x2go_exceptions.X2GoSFTPClientException('failed to initialize SFTP channel')
             try:
                 remote_fileobj = self.sftp_client.open(remote_path, 'w')
                 self.logger('sFTP-write: writing content: %s' % content, loglevel=log.loglevel_DEBUG_SFTPXFER)
@@ -348,7 +363,10 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         self._transport_lock.acquire()
         if ssh_transport and ssh_transport.is_authenticated():
             self.logger('sFTP-write: removing remote file %s on host %s' % (remote_path, self.remote_peername()), loglevel=log.loglevel_DEBUG)
-            self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
+            try:
+                self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
+            except paramiko.SFTPError:
+                raise x2go_exceptions.X2GoSFTPClientException('failed to initialize SFTP channel')
             try:
                 self.sftp_client.remove(remote_path)
             except (x2go_exceptions.SSHException, socket.error, IOError):
@@ -386,7 +404,8 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         else:
             cmd = cmd_line
 
-        cmd = 'sh -c \"%s\"' % cmd
+        cmd_uuid = str(uuid.uuid1())
+        cmd = 'echo X2GODATABEGIN:%s; sh -c \"%s\"; echo X2GODATAEND:%s' % (cmd_uuid, cmd, cmd_uuid)
 
         if self.session_died:
             self.logger("control session seams to be dead, not executing command ,,%s'' on X2Go server %s" % (_rerewrite_blanks(cmd), self.profile_name,), loglevel=loglevel)
@@ -395,6 +414,7 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         self._transport_lock.acquire()
 
         _retval = None
+        _password = None
 
         ssh_transport = self.get_transport()
         if ssh_transport and ssh_transport.is_authenticated():
@@ -404,7 +424,9 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
             timer.start()
             try:
                 self.logger("executing command on X2Go server ,,%s'': %s" % (self.profile_name, _rerewrite_blanks(cmd)), loglevel=loglevel)
-                _retval = self.exec_command(_rewrite_password(cmd, user=self.get_transport().get_username(), password=self._session_password), **kwargs)
+                if self._session_password:
+                    _password = base64.b64decode(self._session_password)
+                _retval = self.exec_command(_rewrite_password(cmd, user=self.get_transport().get_username(), password=_password), **kwargs)
             except AttributeError:
                 self.session_died = True
                 self._transport_lock.release()
@@ -443,6 +465,24 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
             raise x2go_exceptions.X2GoControlSessionException('the X2Go control session is not connected')
 
         self._transport_lock.release()
+
+        # sanitized X2Go relevant data, protect against data injection via .bashrc files
+        (_stdin, _stdout, _stderr) = _retval
+        raw_stdout = _stdout.read()
+
+        sanitized_stdout = ''
+        is_x2go_data = False
+        for line in raw_stdout.split('\n'):
+            if line.startswith('X2GODATABEGIN:'+cmd_uuid): 
+                is_x2go_data = True
+                continue
+            if not is_x2go_data: continue
+            if line.startswith('X2GODATAEND:'+cmd_uuid): break
+            sanitized_stdout += line + "\n"
+
+        _stdout_new = cStringIO.StringIO(sanitized_stdout)
+
+        _retval = (_stdin, _stdout_new, _stderr)
         return _retval
 
     @property
@@ -658,11 +698,11 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
 
         return checkhosts.check_ssh_host_key(self, hostname, port=port)
 
-    def connect(self, hostname, port=22, username='', password='', pkey=None,
+    def connect(self, hostname, port=22, username=None, password=None, passphrase=None, pkey=None,
                 key_filename=None, timeout=None, allow_agent=False, look_for_keys=False,
-                use_sshproxy=False, sshproxy_host='', sshproxy_port=22, sshproxy_user='', sshproxy_password='', sshproxy_force_password_auth=False,
-                sshproxy_key_filename='', sshproxy_pkey=None, sshproxy_look_for_keys=False, sshproxy_allow_agent=False,
-                sshproxy_tunnel='',
+                use_sshproxy=False, sshproxy_host=None, sshproxy_port=22, sshproxy_user=None, sshproxy_password=None, sshproxy_force_password_auth=False,
+                sshproxy_key_filename=None, sshproxy_pkey=None, sshproxy_look_for_keys=False, sshproxy_passphrase='', sshproxy_allow_agent=False,
+                sshproxy_tunnel=None,
                 forward_sshagent=None,
                 unique_hostkey_aliases=None,
                 session_instance=None,
@@ -700,6 +740,10 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         @param password: a password to use for authentication or for unlocking
             a private key
         @type password: C{str}
+        @param passphrase: a passphrase to use for unlocking
+            a private key in case the password is already needed for two-factor
+            authentication
+        @type passphrase: C{str}
         @param key_filename: the filename, or list of filenames, of optional
             private key(s) to try for authentication
         @type key_filename: C{str} or list(str)
@@ -739,6 +783,10 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         @param sshproxy_password: a password to use for SSH proxy authentication or for unlocking
             a private key
         @type sshproxy_password: C{str}
+        @param sshproxy_passphrase: a passphrase to use for unlocking
+            a private key needed for the SSH proxy host in case the sshproxy_password is already needed for
+            two-factor authentication
+        @type sshproxy_passphrase: C{str}
         @param sshproxy_force_password_auth: enforce using a given C{sshproxy_password} even if a key(file) is given
         @type sshproxy_force_password_auth: C{bool}
         @param sshproxy_key_filename: local file location of the private key file
@@ -770,12 +818,25 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         """
         _fake_hostname = None
 
+        if type(password) not in (types.StringType, types.UnicodeType):
+            password = ''
+        if type(sshproxy_password) not in (types.StringType, types.UnicodeType):
+            sshproxy_password = ''
+
         if unique_hostkey_aliases is not None:
             self.unique_hostkey_aliases = unique_hostkey_aliases
             # prep the fake hostname with the real hostname, so we trigger the corresponding code path in 
             # x2go.checkhosts and either of its missing host key policies
             if self.unique_hostkey_aliases:
-                _fake_hostname = "[%s]:%s" % (hostname, port)
+                if port != 22: _fake_hostname = "[%s]:%s" % (hostname, port)
+                else: _fake_hostname = hostname
+
+        if look_for_keys:
+            key_filename = None
+            pkey = None
+
+        _twofactorauth = False
+        if password and (passphrase is None): passphrase = password
 
         if use_sshproxy and sshproxy_host and sshproxy_user:
             try:
@@ -787,6 +848,7 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
                                                               sshproxy_port=sshproxy_port,
                                                               sshproxy_user=sshproxy_user,
                                                               sshproxy_password=sshproxy_password,
+                                                              sshproxy_passphrase=sshproxy_passphrase,
                                                               sshproxy_force_password_auth=sshproxy_force_password_auth,
                                                               sshproxy_key_filename=sshproxy_key_filename,
                                                               sshproxy_pkey=sshproxy_pkey,
@@ -800,7 +862,8 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
                 port = self.sshproxy_session.get_local_proxy_port()
                 _fake_hostname = self.sshproxy_session.get_remote_host()
                 _fake_port = self.sshproxy_session.get_remote_port()
-                _fake_hostname = "[%s]:%s" % (_fake_hostname, _fake_port)
+                if _fake_port != 22:
+                    _fake_hostname = "[%s]:%s" % (_fake_hostname, _fake_port)
 
             except:
                 if self.sshproxy_session:
@@ -822,11 +885,6 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         if add_to_known_hosts:
             self.set_missing_host_key_policy(checkhosts.X2GoAutoAddPolicy(caller=self, session_instance=session_instance, fake_hostname=_fake_hostname))
 
-        # disable pub/priv key authentication if forced
-        if force_password_auth:
-            key_filename = None
-            pkey = None
-
         # trailing whitespace tolerance in hostname
         hostname = hostname.strip()
 
@@ -846,44 +904,85 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
         if timeout and self.low_latency:
             timeout = timeout * 2
 
+        if key_filename and "~" in key_filename:
+            key_filename = os.path.expanduser(key_filename)
+
         if key_filename or pkey or look_for_keys or allow_agent or (password and force_password_auth):
             try:
                 if password and force_password_auth:
-                    self.logger('trying keyboard-interactive SSH authentication with server', loglevel=log.loglevel_DEBUG)
-                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=None, password=password,
+                    self.logger('trying password based SSH authentication with server', loglevel=log.loglevel_DEBUG)
+                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password,pkey=None, 
                                                key_filename=None, timeout=timeout, allow_agent=False,
                                                look_for_keys=False)
                 elif (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
                     self.logger('trying SSH pub/priv key authentication with server', loglevel=log.loglevel_DEBUG)
                     paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=pkey,
-                                               key_filename=key_filename, timeout=timeout, allow_agent=allow_agent,
-                                               look_for_keys=look_for_keys)
+                                               key_filename=key_filename, timeout=timeout, allow_agent=False,
+                                               look_for_keys=False)
                 else:
                     self.logger('trying SSH key discovery or agent authentication with server', loglevel=log.loglevel_DEBUG)
+                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=None,
+                                               key_filename=None, timeout=timeout, allow_agent=allow_agent,
+                                               look_for_keys=look_for_keys)
+
+            except (paramiko.PasswordRequiredException, paramiko.SSHException), e:
+                self.close()
+                if type(e) == paramiko.SSHException and str(e).startswith('Two-factor authentication requires a password'):
+                    self.logger('X2Go Server requests two-factor authentication', loglevel=log.loglevel_NOTICE)
+                    _twofactorauth = True
+                    raise e
+                if passphrase is not None:
+                    self.logger('unlock SSH private key file with provided password', loglevel=log.loglevel_INFO)
                     try:
-                        paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=None,
-                                                      key_filename=None, timeout=timeout, allow_agent=allow_agent,
-                                                      look_for_keys=look_for_keys)
-                    except paramiko.SSHException, e:
-                        if str(e) == 'No authentication methods available':
+                        if not password: password = None
+                        if (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
+                            self.logger('re-trying SSH pub/priv key authentication with server', loglevel=log.loglevel_DEBUG)
+                            try:
+                                paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password, passphrase=passphrase, pkey=pkey,
+                                                           key_filename=key_filename, timeout=timeout, allow_agent=False,
+                                                           look_for_keys=False)
+                            except TypeError:
+                                if _twofactorauth and password and passphrase and password != passphrase:
+                                    self.logger('your version of Paramiko/SSH does not support authentication workflows which require SSH key decryption in combination with two-factor authentication', loglevel=log.loglevel_WARNING)
+                                paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password, pkey=pkey,
+                                                           key_filename=key_filename, timeout=timeout, allow_agent=False,
+                                                           look_for_keys=False)
+                        else:
+                            self.logger('re-trying SSH key discovery now with passphrase for unlocking the key(s)', loglevel=log.loglevel_DEBUG)
+                            try:
+                                paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password, passphrase=passphrase, pkey=None,
+                                                           key_filename=None, timeout=timeout, allow_agent=allow_agent,
+                                                           look_for_keys=look_for_keys)
+                            except TypeError:
+                                if _twofactorauth and password and passphrase and password != passphrase:
+                                    self.logger('your version of Paramiko/SSH does not support authentication workflows which require SSH key decryption in combination with two-factor authentication', loglevel=log.loglevel_WARNING)
+                                paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password, pkey=None,
+                                                           key_filename=None, timeout=timeout, allow_agent=allow_agent,
+                                                           look_for_keys=look_for_keys)
+
+                    except paramiko.AuthenticationException, auth_e:
+                        # the provided password cannot be used to unlock any private SSH key file (i.e. wrong password)
+                        raise paramiko.AuthenticationException(str(auth_e))
+
+                    except paramiko.SSHException, auth_e:
+                        if str(auth_e) == 'No authentication methods available':
                             raise paramiko.AuthenticationException('Interactive password authentication required!')
                         else:
                             self.close()
                             if self.sshproxy_session:
                                 self.sshproxy_session.stop_thread()
-                            raise(e)
+                            raise auth_e
 
-                # since Paramiko 1.7.7.1 there is compression available, let's use it if present...
-                t = self.get_transport()
-                if x2go._paramiko.PARAMIKO_FEATURE['use-compression']:
-                    t.use_compression(compress=True)
-                # enable keep alive callbacks
-                t.set_keepalive(5)
+                else:
+                    self.close()
+                    if self.sshproxy_session:
+                        self.sshproxy_session.stop_thread()
+                    raise e
 
             except paramiko.AuthenticationException, e:
                 self.close()
                 if password:
-                    self.logger('next auth mechanism we\'ll try is keyboard-interactive authentication', loglevel=log.loglevel_DEBUG)
+                    self.logger('next auth mechanism we\'ll try is password authentication', loglevel=log.loglevel_DEBUG)
                     try:
                         paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password,
                                                    key_filename=None, pkey=None, timeout=timeout, allow_agent=False, look_for_keys=False)
@@ -896,7 +995,16 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
                     self.close()
                     if self.sshproxy_session:
                         self.sshproxy_session.stop_thread()
-                    raise(e)
+                    raise e
+
+            except paramiko.SSHException, e:
+                if str(e) == 'No authentication methods available':
+                    raise paramiko.AuthenticationException('Interactive password authentication required!')
+                else:
+                    self.close()
+                    if self.sshproxy_session:
+                        self.sshproxy_session.stop_thread()
+                    raise e
 
             except:
                 self.close()
@@ -909,20 +1017,20 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
             # create a random password if password is empty to trigger host key validity check
             if not password:
                 password = "".join([random.choice(string.letters+string.digits) for x in range(1, 20)])
-            self.logger('performing SSH keyboard-interactive authentication with server', loglevel=log.loglevel_DEBUG)
-            try:
-                paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password,
+            self.logger('performing SSH password authentication with server', loglevel=log.loglevel_DEBUG)
+            #try:
+            paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password,
                                            timeout=timeout, allow_agent=False, look_for_keys=False)
-            except paramiko.AuthenticationException, e:
-                self.close()
-                if self.sshproxy_session:
-                    self.sshproxy_session.stop_thread()
-                raise e
-            except:
-                self.close()
-                if self.sshproxy_session:
-                    self.sshproxy_session.stop_thread()
-                raise
+            #except paramiko.AuthenticationException, e:
+            #    self.close()
+            #    if self.sshproxy_session:
+            #        self.sshproxy_session.stop_thread()
+            #    raise e
+            #except:
+            #    self.close()
+            #    if self.sshproxy_session:
+            #        self.sshproxy_session.stop_thread()
+            #    raise
 
         self.set_missing_host_key_policy(paramiko.RejectPolicy())
 
@@ -935,9 +1043,19 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
 
         # mark Paramiko/SSH transport as X2GoControlSession
         ssh_transport._x2go_session_marker = True
-        self._session_password = password
+        try:
+            self._session_password = base64.b64encode(password)
+        except TypeError:
+            self._session_password = None
 
         if ssh_transport is not None:
+
+            # since Paramiko 1.7.7.1 there is compression available, let's use it if present...
+            if x2go._paramiko.PARAMIKO_FEATURE['use-compression']:
+                ssh_transport.use_compression(compress=True)
+            # enable keep alive callbacks
+            ssh_transport.set_keepalive(5)
+
             self.session_died = False
             self.query_server_features(force=True)
             if self.forward_sshagent:
@@ -1013,7 +1131,10 @@ class X2GoControlSessionSTDOUT(paramiko.SSHClient):
             self.agent_handler.close()
 
         if self.agent_chan is not None:
-            self.agent_chan.close()
+            try:
+                self.agent_chan.close()
+            except EOFError:
+                pass
 
         retval = False
         try:
