@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -529,13 +529,18 @@ class X2GoTerminalSessionSTDOUT(object):
                 ###
                 ### PULSEAUDIO
                 ###
+                cookie_filepath = None
                 if os.path.exists(os.path.normpath('%s/.pulse-cookie' % _LOCAL_HOME)):
+                    cookie_filepath = os.path.normpath('%s/.pulse-cookie' % _LOCAL_HOME)
+                elif os.path.exists(os.path.normpath('%s/.config/pulse/cookie' % _LOCAL_HOME)):
+                    cookie_filepath = os.path.normpath('%s/.config/pulse/cookie' % _LOCAL_HOME)
+                if cookie_filepath is not None:
                     # setup pulse client config file on X2Go server
                     cmd_line = "echo 'default-server=127.0.0.1:%s'>%s/.pulse-client.conf;" % (self.session_info.snd_port, self.session_info.remote_container) + \
                                "echo 'cookie-file=%s/.pulse-cookie'>>%s/.pulse-client.conf" % (self.session_info.remote_container, self.session_info.remote_container)
                     (stdin, stdout, stderr) = self.control_session._x2go_exec_command(cmd_line)
 
-                    self.control_session._x2go_sftp_put(local_path='%s/.pulse-cookie' % _LOCAL_HOME, remote_path='%s/.pulse-cookie' % self.session_info.remote_container)
+                    self.control_session._x2go_sftp_put(local_path=cookie_filepath, remote_path='%s/.pulse-cookie' % self.session_info.remote_container)
 
                     # start reverse SSH tunnel for pulse stream
                     _tunnel = rforward.X2GoRevFwTunnel(server_port=self.session_info.snd_port, 
@@ -1044,6 +1049,7 @@ class X2GoTerminalSessionSTDOUT(object):
             window = utils.find_session_window(self.session_info.name)
 
             if window is not None:
+                self.logger('Session window ID for session %s is: %s' % (self.session_info.name, window.id), loglevel=log.loglevel_DEBUG)
                 self.session_window = window
                 break
 
@@ -1084,6 +1090,7 @@ class X2GoTerminalSessionSTDOUT(object):
             timeout -= 1
 
             if self.session_window is not None:
+                self.logger('Setting session window title for session %s is: %s' % (self.session_info.name, self.session_title), loglevel=log.loglevel_DEBUG)
                 utils.set_session_window_title(self.session_window, self.session_title)
                 break
 
@@ -1379,6 +1386,7 @@ class X2GoTerminalSessionSTDOUT(object):
         @rtype: C{bool}
 
         @raise X2GoTerminalSessionException: if the session startup failed
+        @raise X2GoDesktopSharingDenied: if desktop sharing fails because of denial by the user running the desktop to be shared
 
         """
         self.params.rewrite_session_type()
@@ -1433,7 +1441,7 @@ class X2GoTerminalSessionSTDOUT(object):
         # if the first line of stdout is a "DEN(Y)" string then we will presume that
         # we tried to use X2Go desktop sharing and the sharing was rejected
         if "ACCESS DENIED" in _stderr and "XSHAD" in _stderr:
-            raise x2go_exceptions.X2GoDesktopSharingException('X2Go desktop sharing has been denied by the remote user')
+            raise x2go_exceptions.X2GoDesktopSharingDenied('X2Go desktop sharing has been denied by the remote user')
 
         try:
             self.session_info.initialize(_stdout,
