@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -79,7 +79,10 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         @rtype: C{str}
 
         """
-        return self.get_hostname().split(':')[0].lstrip('[').rstrip(']')
+        if ":" in self.get_hostname():
+            return self.get_hostname().split(':')[0].lstrip('[').rstrip(']')
+        else:
+            return self.get_hostname().lstrip('[').rstrip(']')
 
     def get_hostname_port(self):
         """\
@@ -89,7 +92,10 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         @rtype: C{str}
 
         """
-        return self.get_hostname().split(':')[1]
+        if ":" in self.get_hostname():
+            return int(self.get_hostname().split(':')[1])
+        else:
+            return 22
 
     def get_key(self):
         """\
@@ -216,7 +222,7 @@ class X2GoInteractiveAddPolicy(X2GoMissingHostKeyPolicy):
                     if self.session_instance.control_session.unique_hostkey_aliases:
                         our_server_key = client._host_keys.get(self.session_instance.get_profile_id(), {}).get(keytype, None)
                         if our_server_key is not None:
-                            self.session_instance.logger('SSH host key verification for SSH-proxied host %s with %s fingerprint ,,%s\'\' succeeded. This host is known by the X2Go session profile ID of profile »%s«.' % (self.fake_hostname, self.get_key_name(), self.get_key_fingerprint_with_colons(), self.session_instance.profile_name), loglevel=log.loglevel_NOTICE)
+                            self.session_instance.logger('SSH host key verification for SSH-proxied host %s with %s fingerprint ,,%s\'\' succeeded. This host is known by the X2Go session profile ID of profile ,,%s\'\'.' % (self.fake_hostname, self.get_key_name(), self.get_key_fingerprint_with_colons(), self.session_instance.profile_name), loglevel=log.loglevel_NOTICE)
                             return
                     else:
                         our_server_key = client._host_keys.get(self.fake_hostname, {}).get(keytype, None)
