@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -29,6 +29,7 @@ import os
 import copy
 import paramiko
 import threading
+import types
 
 import string
 import random
@@ -54,11 +55,11 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
     """
     fw_tunnel = None
 
-    def __init__(self, hostname=None, port=22, username=None, password=None, force_password_auth=False, key_filename=None,
+    def __init__(self, hostname=None, port=22, username=None, password=None, passphrase=None, force_password_auth=False, key_filename=None,
                  local_host='localhost', local_port=22022, remote_host='localhost', remote_port=22,
                  known_hosts=None, add_to_known_hosts=False, pkey=None, look_for_keys=False, allow_agent=False,
                  sshproxy_host=None, sshproxy_port=22, sshproxy_user=None,
-                 sshproxy_password=None, sshproxy_force_password_auth=False, sshproxy_key_filename=None, sshproxy_pkey=None,
+                 sshproxy_password=None, sshproxy_force_password_auth=False, sshproxy_key_filename=None, sshproxy_pkey=None, sshproxy_passphrase=None,
                  sshproxy_look_for_keys=False, sshproxy_allow_agent=False,
                  sshproxy_tunnel=None,
                  ssh_rootdir=os.path.join(_LOCAL_HOME, _X2GO_SSH_ROOTDIR), 
@@ -74,6 +75,10 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         @param password: user's password on the SSH proxy host, with private key authentication it will be
             used to unlock the key (if needed)
         @type password: C{str}
+        @param passphrase: a passphrase to use for unlocking
+            a private key in case the password is already needed for two-factor
+            authentication
+        @type passphrase: {str}
         @param key_filename: name of a SSH private key file
         @type key_filename: C{str}
         @param pkey: a private DSA/RSA key object (as provided by Paramiko/SSH)
@@ -108,6 +113,8 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         @type sshproxy_user: C{str}
         @param sshproxy_password: alias for C{password}
         @type sshproxy_password: C{str}
+        @param sshproxy_passphrase: alias for C{passphrase}
+        @type sshproxy_passphrase: C{str}
         @param sshproxy_key_filename: alias for C{key_filename}
         @type sshproxy_key_filename: C{str}
         @param sshproxy_pkey: alias for C{pkey}
@@ -159,6 +166,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
 
         if sshproxy_user: self.username = sshproxy_user
         if sshproxy_password: password = sshproxy_password
+        if sshproxy_passphrase: passphrase = sshproxy_passphrase
         if sshproxy_force_password_auth: force_password_auth = sshproxy_force_password_auth
         if sshproxy_key_filename: key_filename = sshproxy_key_filename
         if sshproxy_pkey: pkey = sshproxy_pkey
@@ -179,6 +187,16 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         self.local_host = self.local_host.strip()
         self.remote_host = self.remote_host.strip()
 
+        # do not use explicitly given keys if look_for_keys has got activated
+        if look_for_keys:
+            key_filename = None
+            pkey = None
+
+        if key_filename and "~" in key_filename:
+            key_filename = os.path.expanduser(key_filename)
+
+        if password and (passphrase is None): passphrase = password
+
         # enforce IPv4 for localhost addresses!!!
         _hostname = self.hostname
         if _hostname in ('localhost', 'localhost.localdomain'):
@@ -191,6 +209,9 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         if username is None:
             username = _CURRENT_LOCAL_USER
 
+        if type(password) not in (types.StringType, types.UnicodeType):
+            password = ''
+
         self._keepalive = True
         self.session_instance = session_instance
 
@@ -226,25 +247,84 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
                                     )
                     elif (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
                         self.connect(_hostname, port=self.port,
-                                     username=self.username, 
+                                     username=self.username,
                                      key_filename=key_filename,
                                      pkey=pkey,
-                                     look_for_keys=look_for_keys,
-                                     allow_agent=allow_agent,
+                                     allow_agent=False,
+                                     look_for_keys=False,
                                     )
                     else:
                         self.connect(_hostname, port=self.port,
-                                     username=self.username, 
+                                     username=self.username,
+                                     key_filename=None,
+                                     pkey=None,
                                      look_for_keys=look_for_keys,
                                      allow_agent=allow_agent,
                                     )
 
-                except x2go_exceptions.AuthenticationException, e:
+                except (paramiko.PasswordRequiredException, paramiko.SSHException), e:
                     self.close()
-                    raise x2go_exceptions.X2GoSSHProxyAuthenticationException('pubkey auth mechanisms both failed')
-                except x2go_exceptions.SSHException, e:
+                    if type(e) == paramiko.SSHException and str(e).startswith('Two-factor authentication requires a password'):
+                        self.logger('SSH proxy host requests two-factor authentication', loglevel=log.loglevel_NOTICE)
+                        raise x2go_exceptions.X2GoSSHProxyException(str(e))
+
+                    if passphrase is None:
+                        try:
+                            if not password: password = None
+                            if (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
+                                try:
+                                    self.connect(_hostname, port=self.port,
+                                                 username=self.username,
+                                                 password=password,
+                                                 passphrase=passphrase,
+                                                 key_filename=key_filename,
+                                                 pkey=pkey,
+                                                 allow_agent=False,
+                                                 look_for_keys=False,
+                                                )
+                                except TypeError:
+                                    self.connect(_hostname, port=self.port,
+                                                 username=self.username,
+                                                 password=passphrase,
+                                                 key_filename=key_filename,
+                                                 pkey=pkey,
+                                                 allow_agent=False,
+                                                 look_for_keys=False,
+                                                )
+                            else:
+                                try:
+                                    self.connect(_hostname, port=self.port,
+                                                 username=self.username,
+                                                 password=password,
+                                                 passphrase=passphrase,
+                                                 key_filename=None,
+                                                 pkey=None,
+                                                 look_for_keys=look_for_keys,
+                                                 allow_agent=allow_agent,
+                                                )
+                                except TypeError:
+                                    self.connect(_hostname, port=self.port,
+                                                 username=self.username,
+                                                 password=passphrase,
+                                                 key_filename=None,
+                                                 pkey=None,
+                                                 look_for_keys=look_for_keys,
+                                                 allow_agent=allow_agent,
+                                                )
+                        except x2go_exceptions.AuthenticationException, auth_e:
+                            raise x2go_exceptions.X2GoSSHProxyAuthenticationException(str(auth_e))
+
+                    else:
+                        if type(e) == paramiko.SSHException:
+                            raise x2go_exceptions.X2GoSSHProxyException(str(e))
+                        elif type(e) == paramiko.PasswordRequiredException:
+                            raise x2go_exceptions.X2GoSSHProxyPasswordRequiredException(str(e))
+                except x2go_exceptions.AuthenticationException:
                     self.close()
-                    raise x2go_exceptions.X2GoSSHProxyAuthenticationException('interactive authentication required')
+                    raise x2go_exceptions.X2GoSSHProxyAuthenticationException('all authentication mechanisms with SSH proxy host failed')
+                except x2go_exceptions.SSHException:
+                    self.close()
+                    raise x2go_exceptions.X2GoSSHProxyAuthenticationException('with SSH proxy host password authentication is required')
                 except:
                     raise
 
@@ -252,6 +332,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
                 t = self.get_transport()
                 if x2go._paramiko.PARAMIKO_FEATURE['use-compression']:
                     t.use_compression(compress=True)
+                t.set_keepalive(5)
 
             # if there is no private key, we will use the given password, if any
             else:
@@ -279,6 +360,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
             self.close()
             raise
 
+
         self.set_missing_host_key_policy(paramiko.RejectPolicy())
         threading.Thread.__init__(self)
         self.daemon = True
