@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -46,8 +46,9 @@ if _X2GOCLIENT_OS != 'Windows':
     from defaults import X_DISPLAY as _X_DISPLAY
 
 if _X2GOCLIENT_OS == 'Windows':
-    import win32api
     import win32gui
+    import win32print
+    import win32con
 
 def is_in_nx3packmethods(method):
 
@@ -555,8 +556,10 @@ def local_color_depth():
             return 24
 
     else:
-        return win32api.GetSystemMetrics(2)
-
+        # This gets the color depth of the primary monitor. All monitors need not have the same color depth.
+        dc = win32gui.GetDC(None)
+        _depth = win32print.GetDeviceCaps(dc, win32con.BITSPIXEL) * win32print.GetDeviceCaps(dc, win32con.PLANES)
+        win32gui.ReleaseDC(None, dc)
 
 def is_color_depth_ok(depth_session, depth_local):
     """\
@@ -602,12 +605,19 @@ def find_session_window(session_name):
         success = False
         windowIDs_obj = root.get_full_property(display.intern_atom('_NET_CLIENT_LIST'), Xlib.X.AnyPropertyType)
 
+        if windowIDs_obj is None:
+            # works with i3 session manager...
+            windowIDs_obj = root.get_full_property(display.intern_atom('_NET_CLIENT_LIST_STACKING'), Xlib.X.AnyPropertyType)
+
         if windowIDs_obj is not None:
             windowIDs = windowIDs_obj.value
 
             for windowID in windowIDs:
                 window = display.create_resource_object('window', windowID)
-                name = window.get_wm_name()
+                try:
+                    name = window.get_wm_name()
+                except Xlib.error.BadWindow:
+                    continue
                 if name is not None and session_name in name:
                     success = True
                     break
