@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2013 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -45,8 +45,10 @@ class X2GoClientPrintingException(_X2GoException): pass
 class X2GoClientSettingsException(_X2GoException): pass
 class X2GoSessionException(_X2GoException): pass
 class X2GoControlSessionException(_X2GoException): pass
+class X2GoSFTPClientException(_X2GoException): pass
 class X2GoRemoteHomeException(_X2GoException): pass
 class X2GoHostKeyException(_X2GoException): pass
+class X2GoSSHProxyPasswordRequiredException(_X2GoException): pass
 class X2GoSSHProxyHostKeyException(_X2GoException): pass
 class X2GoTerminalSessionException(_X2GoException): pass
 class X2GoSessionCacheException(_X2GoException): pass
@@ -64,7 +66,8 @@ class X2GoMIMEboxQueueException(_X2GoException): pass
 class X2GoSSHProxyException(_X2GoException): pass
 class X2GoSSHProxyAuthenticationException(_X2GoException): pass
 class X2GoNotImplementedYetException(_X2GoException): pass
-class X2GoDesktopSharingException(_X2GoException): pass
+class X2GoDesktopSharingDenied(_X2GoException): pass
+class X2GoDesktopSharingDenied(_X2GoException): pass
 class X2GoTimeOutException(_X2GoException): pass
 if _X2GOCLIENT_OS != 'Windows':
     # faking Windows errors on non-Windows systems...
